"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Cpk = void 0;
const cache_1 = __importDefault(require("./src/cache"));
const functions_1 = require("./src/functions");
const Providers = {};
class Cpk {
    /**
     * ===============================================
     * Instance Methods
     * ===============================================
     */
    constructor(providerName) {
        this.useCache = true;
        providerName = providerName.toLowerCase();
        if (!Providers[providerName]) {
            throw new Error(`Provider (${providerName}) not found`);
        }
        if (this.useCache) {
            this.cache = new cache_1.default({
                prefix: providerName
            });
        }
        this.provider = Providers[providerName];
    }
    static useProviders(providers = []) {
        // Loop and add providers
        for (const fn of providers) {
            if (typeof fn === "function") {
                const provider = fn();
                Providers[provider.name] = provider;
            }
            else {
                Providers[fn.name] = fn;
            }
        }
        return this;
    }
    /**
     * Disable useCache
     */
    disableCache() {
        this.useCache = false;
        this.cache = undefined;
        return this;
    }
    /**
     * Get a single pair
     * @param pair
     * @param ttl
     */
    async get(pair, ttl = 60) {
        const [coin, currency] = this.parsePair(pair);
        // if cache is enabled, check cache first
        if (this.cache)
            return this.cache.getOrSetAsync((0, functions_1.pairString)(coin, currency), () => this.provider.getPrice(coin, currency), ttl);
        return this.provider.getPrice(coin, currency);
    }
    async getMany(pairs, ttl = 60) {
        let results = {};
        const pairsNotInCache = [];
        // if cache is enabled, check cache first
        if (this.cache) {
            for (const pair of pairs) {
                const [, , paired] = this.parsePair(pair);
                if (this.cache.has(paired)) {
                    results[paired] = this.cache.get(paired);
                }
                else {
                    pairsNotInCache.push(paired);
                }
            }
        }
        const data = [];
        for (const pair of this.cache ? pairsNotInCache : pairs) {
            const [coin, currency] = this.parsePair(pair);
            data.push({ coin, currency });
        }
        if (data.length) {
            const prices = await this.provider.getPrices(data);
            if (this.cache) {
                for (const pair of Object.keys(prices))
                    this.cache.set(pair, prices[pair], ttl);
            }
            results = Object.assign(results, prices);
        }
        return results;
    }
    /**
     * ===============================================
     * Private Methods
     * ===============================================
     */
    /***
     * Parse a pair of string into an array
     * This function also validates the pair
     * Checks if the coin/currency is supported
     * @param pair
     * @private
     */
    parsePair(pair) {
        pair = pair.toUpperCase();
        let [coin, currency] = pair.split("/");
        if (!currency)
            currency = "USD";
        // Check if coin is in the provider
        if (this.provider.coinsSupported !== "any" &&
            !this.provider.coinsSupported.includes(coin)) {
            throw new Error(`Coin (${coin}) not supported in provider: ${this.provider.name}`);
        }
        // Check if currency is in the provider
        if (this.provider.coinsSupported !== "any" &&
            !this.provider.currenciesSupported.includes(currency)) {
            throw new Error(`Currency (${currency}) not supported in provider: ${this.provider.name}`);
        }
        // check if coin exists in the provider
        return [coin, currency, (0, functions_1.pairString)(coin, currency)];
    }
}
exports.Cpk = Cpk;
